/*********************************************************************
* Copyright (C) 2006 by Progress Software Corporation. All rights    *
* reserved.  Prior versions of this work may contain portions        *
* contributed by participants of Possenet.                           *
*                                                                    *
*********************************************************************/
/**  DataSource manages the source query for a target and deals with various 
     query tranformation in support of this. It implements IQueryMap 
     to utilize QueryString for the transformation                       HD **/

using com.openedge.framework.util.query.*.
    
CLASS com.openedge.framework.util.query.DataSource INHERITS DBQuery IMPLEMENTS IQueryMap:
  
  DEFINE PROTECTED VARIABLE DataSourceHandle  AS HANDLE NO-UNDO.
  DEFINE PROTECTED VARIABLE DataBuffer        AS HANDLE  NO-UNDO.
  
  DEFINE PRIVATE   VARIABLE mcValueDelimiter  AS CHARACTER  NO-UNDO.
  DEFINE PRIVATE   VARIABLE mcClientPosition  AS CHAR       NO-UNDO.
  DEFINE PRIVATE   VARIABLE mcPrevPosition    AS CHAR       NO-UNDO.
  DEFINE PRIVATE   VARIABLE mcClientRequest   AS CHAR       NO-UNDO.
  
  DEFINE PRIVATE   VARIABLE mlIgnoreLastBatch AS LOGICAL    NO-UNDO.

  /* the queryString class would need to handlke db options etc..  
  DEFINE PROTECTED VARIABLE SourceQueryString AS CLASS query.QueryString.
   */

  CONSTRUCTOR PUBLIC DataSource (phBuffer   AS HANDLE,
                                 pcTables   AS CHAR,
                                 pcMapping  AS CHAR   ) :

    SUPER(pcTables,''). 
    mcValueDelimiter = CHR(1).
    DataBuffer = phBuffer.
    CREATE DATA-SOURCE DataSourcehandle. 
    DataSourceHandle:QUERY = QueryHandle.
    phBuffer:ATTACH-DATA-SOURCE(DataSourceHandle:HANDLE,pcMapping).
  END CONSTRUCTOR.

  DESTRUCTOR PUBLIC DataSource (  ) :
    /* this is probably not needed as it should be in the widget pool */
    DELETE OBJECT DataSourceHandle NO-ERROR.
  END DESTRUCTOR.

  /********  DataBuffer  **********/
  /* The buffer is currently considered an outsider in the sense that 
     batch size is set externally.
     This is for internal use and is NOT
     -- public getRequestedBatchSize 
     -- public getNumRows
  */ 

  METHOD PROTECTED INTEGER getBatchSize ():
    RETURN DataBuffer:BATCH-SIZE.
  END METHOD.
  
  METHOD PROTECTED LOGICAL setBatchSize (piSize AS INT):
    DataBuffer:BATCH-SIZE = piSize.
    RETURN TRUE.
  END METHOD.

  /* init */
  METHOD PRIVATE VOID initialize():
    ASSIGN
      mcPrevPosition    = ''
      mcClientPosition  = '' 
      mlIgnoreLastBatch = FALSE.
    resetQuery().
  END METHOD.

  /* prepare a client query for Fill */ 
  METHOD PUBLIC LOGICAL prepareForFill (pcTargetQuery AS CHAR,pcJoin AS CHAR):
  
    DEFINE VARIABLE oQueryString AS CLASS QueryString.
    DEFINE VARIABLE lOk          AS LOGICAL    NO-UNDO.
    DEFINE VARIABLE hParentRel   AS HANDLE     NO-UNDO.
    
    initialize(). /*  clean up */

    hParentRel = DataBuffer:PARENT-RELATION.
    oQueryString = NEW QueryString(pcTargetQuery,THIS-OBJECT).
    IF pcJoin > '' THEN
      oQueryString:addExpression(pcJoin).
    /* Data-Sources are defined with queries in the constructor in the event that
       there is a join involved. Add and transform the fill-where-string for the dependent 
       tables so that Progress can identify the related when filling the temp-tables.
      (columnValue ensures parent reference is NOT tranformed) */
    ELSE IF VALID-HANDLE(hParentRel) AND hParentRel:ACTIVE AND NOT hParentRel:REPOSITION THEN
       oQueryString:addExpression(DataSourceHandle:FILL-WHERE-STRING).
    
    lOk = prepareQueryString(oQueryString).
   
    DELETE OBJECT oQueryString. 
    RETURN lOk.
  END METHOD. 

  /* prepare with QueryString (subclass override this for query optimization) */
  METHOD PUBLIC LOGICAL prepareQueryString (poQueryString AS CLASS QueryString):    
    DEFINE VARIABLE lOk AS LOGICAL    NO-UNDO.
    
    IF VALID-OBJECT(poQueryString) THEN
      lok = poQueryString:insertToQuery(QueryHandle).
    ELSE 
      lok = prepare().
    RETURN lok.
  END METHOD.
  
  /* set start rowid to where position */
  METHOD PUBLIC LOGICAL positionWhere (pcWhere AS CHAR, plFill AS LOGICAL):
    
    DEFINE VARIABLE oQueryString AS Querystring NO-UNDO.
    DEFINE VARIABLE cBaseQuery   AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE cPosition    AS CHARACTER  NO-UNDO.

    /* querytring does not handle  a clean where string (yet?), 
       so add a for each (buffer name is irrelevant for transform source) */ 
    IF NOT (pcWhere BEGINS 'FOR ') THEN
      oQueryString = NEW QueryString
         ('FOR EACH dummy ' + pcwhere,THIS-OBJECT).
    cBaseQuery = QueryHandle:PREPARE-STRING.
    /* prepare and open with find criteria */
    oQueryString:insertToQuery(QueryHandle).  
    DELETE OBJECT oQueryString.
    QueryHandle:QUERY-OPEN.
    QueryHandle:GET-FIRST.
    /* IMPORTANT to set query back,  a sorted query could remain with 
       criteria and relieve the datasource of the restart-rowid, but  
       we also need this for fill and prev position */ 
    QueryHandle:QUERY-PREPARE(cBaseQuery).
    cPosition = getPosition().
    IF cPosition > '' THEN
    DO:
      mcClientPosition = getKeyValue().
      setStartFromQuery(QueryHandle).
      QueryHandle:QUERY-OPEN.
      setPosition(cPosition).
      /* Check and set prev position */
      QueryHandle:GET-PREV.
      mcPrevPosition = getPosition(). /* blank if not avail*/
      IF plFill THEN
      DO:
        QueryHandle:GET-NEXT.
        IF NOT QueryHandle:REPOSITION-FORWARDS(getBatchSize()) THEN
          positionToLastBatch().
      END.
    END. /* position > '' */
    QueryHandle:QUERY-CLOSE.
    RETURN cPosition > ''.
  END METHOD.


  /* set start rowid to get one batch including last*/
  METHOD PUBLIC LOGICAL positionToLastBatch():
    DEFINE VARIABLE lIsOpen AS LOGICAL    NO-UNDO.
    lIsOpen = QueryHandle:IS-OPEN.
    IF NOT lIsOpen THEN 
      QueryHandle:QUERY-OPEN().
    QueryHandle:GET-LAST().
    mcClientPosition = getKeyValue().
    QueryHandle:REPOSITION-BACKWARDS(getBatchSize()).
    QueryHandle:GET-NEXT.
    setStartFromQuery(QueryHandle).
    QueryHandle:GET-PREV.   /* more records on top?  */
    mcPrevPosition = getPosition().
    IF NOT lIsOpen THEN 
      QueryHandle:QUERY-CLOSE().
    setBatchSize(0). /* we want the rest of the data */
    RETURN TRUE. /* should return false if not found (fill would be empty though) */
  END METHOD.

  /* are there more records ahead?   */
  METHOD PUBLIC CHARACTER getNextPosition ():
    DEFINE VARIABLE cBuffer    AS CHARACTER NO-UNDO.
    DEFINE VARIABLE iBuffer    AS INTEGER    NO-UNDO.
    DEFINE VARIABLE cPosition  AS CHARACTER  NO-UNDO.

    /* ignore if batch was set to append to this record (setPrevPosition) */
    IF NOT mlIgnoreLastBatch AND DataBuffer:LAST-BATCH = NO THEN
    DO iBuffer = 1 TO NUM-ENTRIES(Tables):
      ASSIGN
        cBuffer = ENTRY(iBuffer,Tables)
        cPosition = cPosition 
                  + (IF iBuffer = 1 THEN '' ELSE mcRowidDlm)
                  +  STRING(DataSourceHandle:NEXT-ROWID(cBuffer)).
    END.
    /* True key positioning would require set position to get the true next key 
       setPosition(cPosition)
       cPosition= getPosition()  */
    RETURN cPosition.
  END METHOD.
   
  METHOD PUBLIC CHARACTER getPrevPosition ():    
    RETURN mcPrevPosition.  
  END METHOD.

  METHOD PUBLIC CHARACTER getClientPosition ():  

    IF mcClientRequest = 'LAST' THEN
      RETURN getLastKeyValue(). 
    IF mcClientPosition = '' THEN
      RETURN getFirstKeyValue().
    
    RETURN mcClientPosition.  
  END METHOD.
  
  METHOD PUBLIC VOID setClientRequest (pcRequest AS CHAR ):    
    mcClientRequest = pcRequest.
  END METHOD.

  /* set position for a request for next batch  */ 
  METHOD PUBLIC LOGICAL setNextPosition (pcPosition AS CHAR):     
    DEFINE VARIABLE lOk AS LOGICAL    NO-UNDO.

    IF pcPosition <> 'FIRST' AND pcPosition > '' THEN
    DO:
      QueryHandle:QUERY-OPEN.
      lOk = setPosition(pcPosition).
      IF lOk THEN
      DO:  
        mcClientPosition = getKeyValue(). 
        setStartFromQuery(QueryHandle).    
      END.
      QueryHandle:QUERY-CLOSE.
      RETURN lok.
    END.
    RETURN TRUE.
    /*  This is faster, but ClientPosition needs the above and cannot 
        capture failed positioning here   
    IF pcPosition <> 'FIRST' AND pcPosition <> ? THEN
    DO iBuffer = 1 TO NUM-ENTRIES(Tables):
      cBuffer = ENTRY(iBuffer,Tables).
      DataSourceHandle:RESTART-ROWID(cBuffer) 
         = TO-ROWID(ENTRY(iBuffer,pcPosition,mcRowidDlm)).
    END.
    RETURN TRUE.
    */
  END METHOD.
  
  /* set position for a request for prev batch             */
  METHOD PUBLIC LOGICAL setPrevPosition (pcPosition AS CHAR):      
    DEFINE VARIABLE iBatch AS INTEGER    NO-UNDO.
    DEFINE VARIABLE lOk    AS LOGICAL    NO-UNDO.

    QueryHandle:QUERY-OPEN().
    lOk = setPosition(pcPosition).
    IF lOk THEN
    DO:
      mcClientPosition = getKeyValue().
      QueryHandle:REPOSITION-BACKWARDS(getBatchSize()).    
      /* At top, so setBatchSize to not read the previous passed in */      
      IF QueryHandle:QUERY-OFF-END THEN 
      DO:
        iBatch = 1.
        DO WHILE TRUE:
          QueryHandle:GET-NEXT.
          IF getPosition() = pcPosition THEN
            LEAVE.   
          iBatch = iBatch + 1.
        END.
        setBatchSize(iBatch).
      END.
      ELSE DO:
        QueryHandle:GET-NEXT.
        setStartFromQuery(QueryHandle).
        QueryHandle:GET-PREV.   /* more records on top?  */
        mcPrevPosition = getPosition().
      END.
      mlIgnoreLastBatch = TRUE. /* set flag to not return nextposition */   
    END. /* if setPosition */
    QueryHandle:QUERY-CLOSE().
    RETURN lOk.
  END METHOD.

  /* set start from current rexord in a query */
  METHOD PRIVATE VOID setStartFromQuery(phQuery AS HANDLE):    
    DEFINE VARIABLE iBuffer   AS INTEGER    NO-UNDO.
    DEFINE VARIABLE hBuffer   AS HANDLE     NO-UNDO.
    DO iBuffer = 1 TO phQuery:NUM-BUFFERS:
      hBuffer = phQuery:GET-BUFFER-HANDLE(iBuffer).
      DataSourceHandle:RESTART-ROWID(hBuffer:NAME) = hBuffer:ROWID. 
    END.
  END METHOD. 

  METHOD PROTECTED CHARACTER getKeyValue():
    DEFINE VARIABLE i       AS INTEGER    NO-UNDO.
    DEFINE VARIABLE cField  AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE cValue  AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE cSource AS CHARACTER  NO-UNDO.
    DO i = 1 TO NUM-ENTRIES(DataBuffer:KEYS):
      ASSIGN
        cField = DataBuffer:NAME + "." + ENTRY(i,DataBuffer:KEYS)
        cSource = columnSource(cField) 
        cValue = (IF i > 1 THEN cValue + mcValueDelimiter ELSE '')
               + cField 
               + mcValueDelimiter
               + columnValue(cSource).
     
    END.
     
    RETURN cValue.
  END METHOD.

  METHOD PROTECTED CHARACTER getFirstKeyValue():
    DEFINE VARIABLE cKeyValue AS CHARACTER  NO-UNDO.
    /* This is called post fill and should probably be replaced by logic that
       finds the record in the filled TT instead of the DB */
       
    IF QueryHandle:PREPARE-STRING <> ? THEN
    DO:
      QueryHandle:QUERY-OPEN.
      QueryHandle:GET-FIRST.
/*      MESSAGE                                   */
/*      QueryHandle:get-buffer-handle(1):name skip*/
/*      QueryHandle:prepare-string skip           */
/*                                                */
/*           VIEW-AS ALERT-BOX.                   */
      cKeyValue = getKeyValue(). 
      QueryHandle:QUERY-CLOSE.
    END.
    RETURN cKeyValue.
  END METHOD.
 
  METHOD PROTECTED CHARACTER getLastKeyValue():
    DEFINE VARIABLE cKeyValue AS CHARACTER  NO-UNDO.
    /* This is called post fill and should probably be replaced by logic that
       finds the record in the filled TT instead of the DB */
    IF QueryHandle:PREPARE-STRING <> ? THEN
    DO:
      QueryHandle:QUERY-OPEN.
      QueryHandle:GET-LAST.
      cKeyValue = getKeyValue(). 
      QueryHandle:QUERY-CLOSE.
    END.
    RETURN cKeyValue.
  END METHOD.
 
  /* return source columns for query mapping 
    (part of IQueryMap interface used by QueryString to map query */
  METHOD PUBLIC CHARACTER columnSource (pcColumn AS CHAR):
    DEFINE VARIABLE iLookup    AS INTEGER    NO-UNDO.
    DEFINE VARIABLE cTable     AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE hBuffer    AS HANDLE     NO-UNDO.
    DEFINE VARIABLE hDataset   AS HANDLE     NO-UNDO.
    DEFINE VARIABLE cColumn    AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE hParentRel AS HANDLE     NO-UNDO.
    /*  inner join reference to other buffers can be passed from client,
        so we use the buffer's dataset to get those datasources' 
        fieldmap. This assume that any buffer that has been attached to a 
        datasource is referencing the same physical storage, 
        (all connected progress databases are the same physical storage in 
        this context), so all dataset mapping is valid for this datasource. 
        Note that this secondary inner join is passed from the client 
        as an option to reduce the number of records to fill to for THIS table. */
                                      
    cTable = ENTRY(1,pcColumn,".").
    IF DataBuffer:NAME = cTable THEN
      hBuffer = DataBuffer.
    /* keep reference to parent relation as is  */
    ELSE DO:
      hParentRel = DataBuffer:PARENT-RELATION.
      IF VALID-HANDLE(hParentRel) AND cTable = hParentRel:PARENT-BUFFER:NAME THEN
        RETURN pcColumn.
    END.
    
    ASSIGN
      hDataset = DataBuffer:DATASET
      hBuffer  = hDataSet:GET-BUFFER-HANDLE(cTable) NO-ERROR.
    
    IF VALID-HANDLE(hBuffer) THEN
    DO:
      iLookup = LOOKUP(pcColumn,hBuffer:DATA-SOURCE-COMPLETE-MAP).
      IF iLookup > 0 THEN
      DO:
        cColumn =  ENTRY(iLookup + 1,hBuffer:DATA-SOURCE-COMPLETE-MAP).
        IF hBuffer <> DataBuffer AND LOOKUP(ENTRY(1,pcColumn,"."),Tables) = 0 THEN
          RETURN pcColumn.
        RETURN cColumn.
      END.
      ELSE
        RETURN pcColumn.
    END.
    
  END METHOD.                   

  /* not in use - could allow an external procedure like the DAO to register call back events */
  METHOD PUBLIC VOID setCallBack(pcEvent   AS CHAR,
                                 pcMethod  AS CHAR,
                                 phTarget  AS HANDLE):
    /*  if 
    DataBuffer:SET-CALLBACK(pcEvent, pcMethod, phTarget).
       else
    DataBuffer:SET-CALLBACK(pcEvent, pcMethod, this-object).
      */
  END METHOD.
END.

