/*************************************************************/
/* Copyright (c) 2007 by Progress Software Corporation      */
/*************************************************************/
/*------------------------------------------------------------------------------
    File   :   adm2/serviceadapter.p
    Purpose:   Data request service adapter interface
    Notes    - This is a sample that illustrates retrieval of definitions only
               Allows assembly of DataViews without a service..
             - This is meant to imply that this is something you ever need
               But one could consider using a dedicated Service for definitions   
               in a Service Adapter 
------------------------------------------------------------------------------*/
/* ************************  Definitions  ********************** */
define stream fileStream.

/* ***********************  Procedures ********************************* */
procedure destroyObject :
/*------------------------------------------------------------------------------
  Purpose:  Destroy     
------------------------------------------------------------------------------*/
  delete procedure this-procedure .
end procedure .

procedure retrieveData :
/*------------------------------------------------------------------------------
  Purpose:  Retrieve data or definitions for one or many Business Entities 
            The caller specifies the number of entities that need to be handled 
            by the request by defining a finite number of extents for the array 
            parameters.   
Parameters:  
     pcEntity       - Business Entity 
     pcTables       - Tables for which to retrieve data or definitions
     pcQueries      - List of queries that corresponds to the requested tables
     pcJoins        - List of join fields for the tables
     pcPositions    - How to find a single record in the tables  
     pcRequest      - The actual request for the tables   
     pcBatchContext - Context info for a batch request 
     plFillBatch    - Read a full batch size at end of result set
     pcNumRecords   - Batch size for the tables               
     phDataSet      - Prodataset handle 
     pcContext      - Other context    
     pcPrevContext  - Context for prev record that was NOT returned from tables
     pcNextContext  - Context for next record that was NOT returned from tables
                                        
  Notes: The comments has been minimized to focus on code.
         (see orignal or whitepaper) 
------------------------------------------------------------------------------*/
 define input         parameter pcEntity       as character no-undo extent.
 define input         parameter pcTables       as character no-undo extent.
 define input         parameter pcQueries      as character no-undo extent.
 define input         parameter pcJoins        as character no-undo extent.
 define input         parameter pcPositions    as character no-undo extent.
 define input         parameter pcRequests     as character no-undo extent.
 define input         parameter pcBatchContext as character no-undo.
 define input         parameter plFillBatch    as logical   no-undo.
 define input-output  parameter pcNumRecords   as character no-undo extent.
 define input-output  parameter phDataset      as handle    no-undo extent. 
 define input-output  parameter pcContext      as character no-undo extent.
 define output        parameter pcPrevContext  as character no-undo extent.
 define output        parameter pcNextContext  as character no-undo extent.
 
  /* dim output from input */
  extent(pcPrevContext) = extent(pcEntity).
  extent(pcNextContext) = extent(pcEntity). 
  
  define variable iEntity  as integer   no-undo.
 
  do iEntity = 1 to extent(pcEntity): 
    if pcTables[iEntity] = '' then
    do:
      run generateDataset (pcEntity[iEntity],output phDataset[iEntity]) no-error. 
      if error-status:error then 
        return error return-value.                                          
    end.
    else 
      return error "Retrieval of data is not implemented in this Service". 
  end.  /* entity loop */     
end procedure /* retrieveData */. 

procedure submitData :
/*------------------------------------------------------------------------------
  Purpose:     Submit of changes
  parameters:  
     pcEntity       - Business Entity 
     phDataSet      - Prodataset handle with changes only 
     pcContext      - Other context    
     Notes:        
------------------------------------------------------------------------------*/
  /* Business Entity reference */
  define input         parameter pcEntity     as character no-undo extent.
  define input-output  parameter phDataset    as handle    no-undo extent.
  define input-output  parameter pcContext    as character no-undo extent.
  
  return error "Submit of data is not implemented in this Service".                              
end procedure .

/* ************************  Function Implementations ***************** */

procedure generateDataset :
/*------------------------------------------------------------------------------
  Purpose:     return a dynamic dataset from an include 
  Parameters:  include name 
               datasetname in include 
           OUT procedurename
  Note:    Hard-coded path and naming convention
------------------------------------------------------------------------------*/
  define input  parameter pcEntity   as  character   no-undo.
  define output parameter phDataset  as  handle      no-undo.
  
  define variable cInclude           as  character   NO-UNDO.
  define variable cFile              as  character   NO-UNDO.
  define variable cDataset           as  character   NO-UNDO.
  
  assign
    cInclude = "schema/ds" + pcEntity + ".i"
    cDataset = "ds" + pcEntity.
  if search(cInclude) = ? then 
    return error 'Could not find ' + cInclude. 
  /* get a unique temp file name in tempdir */
  run adecomm/_tmpfile.p('','.p', output cFile).
  /* write the file */
  output stream FileStream to value(cFile) no-map .
  put stream FileStream unformatted  
    "~{" cInclude "~}" SKIP
    "define output parameter dataset for " cDataSet   " .".
  output stream FileStream close.
  do on stop undo,leave:
    run value(cFile) (output dataset-handle phDataset).
  end.   
  os-delete value(cFile) no-error.
end procedure.

FUNCTION getObjectType RETURNS CHARACTER
  (  ) :
/*------------------------------------------------------------------------------
  Purpose: To be used by designtime super destroy  
    Notes:  
------------------------------------------------------------------------------*/
  RETURN "Service". 
END FUNCTION.


