/*************************************************************/
/* Copyright (c) 2007 by Progress Software Corporation      */
/*************************************************************/
/*------------------------------------------------------------------------------
    File   :   adm2/serviceadapter.p
    Purpose:   Data request service adapter interface
       Note:   This sample uses a very limited service that just returns a 
               completely filled proDataset. 
             - The dataview query will work on client data and depends on
               the dataset not using batching.       
             - Will not work with REPOSITION tables if more than one 
               request is issued.
             - The adapter is responsible of starting and closing the service      
------------------------------------------------------------------------------*/
/* ************************  Definitions  ********************** */  
define variable ghService as handle no-undo.

/* ************************  Function Prototypes ********************** */
function getObjectType returns character 
  (  )  forward .

function getService returns handle private 
  (  )  forward .

function destroyService returns logical private 
  (  )  forward .

/* ***********************  Procedures ********************************* */
procedure destroyObject :
/*------------------------------------------------------------------------------
  Purpose:  Destroy     
------------------------------------------------------------------------------*/
  destroyService(). 
  delete procedure this-procedure .
end procedure .

procedure retrieveData :
/*------------------------------------------------------------------------------
  Purpose:  Retrieve data or definitions for one or many Business Entities 
            The caller specifies the number of entities that need to be handled 
            by the request by defining a finite number of extents for the array 
            parameters.   
Parameters:  
     pcEntity       - Business Entity 
     pcTables       - Tables for which to retrieve data or definitions
     pcQueries      - List of queries that corresponds to the requested tables
     pcJoins        - List of join fields for the tables
     pcPositions    - How to find a single record in the tables  
     pcRequest      - The actual request for the tables   
     pcBatchContext - Context info for a batch request 
     plFillBatch    - Read a full batch size at end of result set
     pcNumRecords   - Batch size for the tables               
     phDataSet      - Prodataset handle 
     pcContext      - Other context    
     pcPrevContext  - Context for prev record that was NOT returned from tables
     pcNextContext  - Context for next record that was NOT returned from tables
                                        
  Notes: The comments has been minimized to focus on code.
         (see orignal or whitepaper) 
------------------------------------------------------------------------------*/
 define input         parameter pcEntity       as character no-undo extent.
 define input         parameter pcTables       as character no-undo extent.
 define input         parameter pcQueries      as character no-undo extent.
 define input         parameter pcJoins        as character no-undo extent.
 define input         parameter pcPositions    as character no-undo extent.
 define input         parameter pcRequests     as character no-undo extent.
 define input         parameter pcBatchContext as character no-undo.
 define input         parameter plFillBatch    as logical   no-undo.
 define input-output  parameter pcNumRecords   as character no-undo extent.
 define input-output  parameter phDataset      as handle    no-undo extent. 
 define input-output  parameter pcContext      as character no-undo extent.
 define output        parameter pcPrevContext  as character no-undo extent.
 define output        parameter pcNextContext  as character no-undo extent.

  /* dim output from input */
  extent(pcPrevContext) = extent(pcEntity).
  extent(pcNextContext) = extent(pcEntity).

  define variable iEntity     as integer   no-undo.
 
  do iEntity = 1 to extent(pcEntity): 
    if pcTables[iEntity] <> '' then     
      run fetchData in getService() (pcEntity[iEntity], output phDataset[iEntity]) no-error.
    else 
      run fetchDataset in getService() (pcEntity[iEntity], output phDataset[iEntity]) no-error.
   
    if error-status:error then 
      return error if return-value > '' then return-value else error-status:get-message(1).                                   
  end.  /* entity loop */        
end procedure /* retrieveData */. 

procedure submitData :
/*------------------------------------------------------------------------------
  Purpose:     Submit of changes
  parameters:  
     pcEntity       - Business Entity 
     phDataSet      - Prodataset handle with changes only 
     pcContext      - Other context    
     Notes:        
------------------------------------------------------------------------------*/
  /* Business Entity reference */
  define input         parameter pcEntity     as character no-undo extent.
  define input-output  parameter phDataset    as handle    no-undo extent.
  define input-output  parameter pcContext    as character no-undo extent.

  /* we don't loop as we know the adm2 does not submit more than one */  
  run saveChanges in getService()(pcEntity[1],
                                  input-output phDataset[1]) no-error. 
  if error-status:error then  
    return error if return-value > '' then return-value else error-status:get-message(1).  
  
  return.                              
end procedure .

/* ************************  Function Implementations ***************** */
FUNCTION getObjectType RETURNS CHARACTER
  (  ) :
/*------------------------------------------------------------------------------
  Purpose: To be used by designtime super destroy  
    Notes:  
------------------------------------------------------------------------------*/
  RETURN "Service". 
END FUNCTION.

/* get service starts on first call  */
function getService returns handle private 
  (  ):
  if not valid-handle(ghService) then 
  do on stop undo, leave:
    run support/service.p persistent set ghService.  
  end.
  return ghService. 
end.

function destroyService returns logical private 
  (  ): 
  if valid-handle(ghService) then 
  do:
    run destroyObject in ghService.
    return true.
  end.   
  return false. 
end.
