/*------------------------------------------------------------------------------
 File        : daOrder order data access   
 Description : Example with no paramters for query or batching and no BL.
 Notes       :           
 ------------------------------------------------------------------------------*/
/* ***************************  Definitions  ************************** */
{schema/dsOrder.i}
/* define data sources */
define data-source dOrder for Order.
define data-source dOrderLine for OrderLine.

/* ************ Functions ****************************************/           
function errorString returns character private 
  (pcString as char):
  if pcString > '' then 
    return pcString.
  else    
    return error-status:get-message(1). 
end.
   
/* ************ Procedures ****************************************/        
procedure fetchData:
/*------------------------------------------------------------------------------
  Purpose: Fetch data from the physical storage.              
Parameters: phDataSet - Prodataset handle (NOT dataset-handle)     
 -----------------------------------------------------------------------------*/
  define output parameter dataset for dsOrder.
  run attachDataSources (input-output dataset dsOrder by-reference).
  dataset dsOrder:fill().
  run detachDataSources (input-output dataset dsOrder by-reference). 
end. /* retrieveData */

procedure attachDataSources:
  define input-output parameter dataset for dsOrder.
  /* this should probably be handled elsewhere typically not an issue on a server 
     but this test is being run with local connection */
  temp-table eOrder:tracking-changes = false. 
  temp-table eOrderLine:tracking-changes = false. 
  buffer eOrder:attach-data-source(data-source dOrder:handle).        
  buffer eOrderLine:attach-data-source(data-source dOrderLine:handle).
end.    

procedure detachDataSources:
  define input-output parameter dataset for dsOrder.
  buffer eOrder:detach-data-source().        
  buffer eOrderLine:detach-data-source().
end.   

procedure saveChanges:
/*------------------------------------------------------------------------------
  Purpose:  Save changes to physical storage                
Parameters: dataset dsOrder 
     Notes: This example of data access saves all in one transaction, 
-----------------------------------------------------------------------------*/
  define input-output parameter dataset for dsOrder.

  run attachDataSources (input-output dataset dsOrder by-reference). 
  /* traverse before records to get deletes */  
  OrderTrans: 
  do transaction:
    /* do orderline deletes first */
    for each eOrderLineBefore where buffer eOrderLineBefore:row-state = row-deleted:             
      buffer eOrderLineBefore:save-row-changes() no-error.
      if buffer eOrderLineBefore:error then 
      do:
        buffer eOrderLineBefore:error-string = errorString(return-value).
        UNDO OrderTrans,LEAVE OrderTrans.
      end.
    end. /* each eorderlinebefore = delete */
      
    for each eOrderBefore:
      buffer eOrderBefore:save-row-changes(1,'ordernum') no-error.      
      if buffer eOrderBefore:error then
      do: 
        buffer eOrderBefore:error-string = errorString(return-value).
        UNDO OrderTrans,LEAVE OrderTrans.
      end.
    end. /* for each eOrderBefore */
    
    /* save orderline changes  */ 
    for each eOrderLineBefore where buffer eOrderLineBefore:row-state <> row-deleted:             
      buffer eOrderLineBefore:save-row-changes() no-error.
      if buffer eOrderLineBefore:error then 
      do:
        buffer eOrderLineBefore:error-string = errorString(return-value).
        UNDO,NEXT.
      end.       
    end. /* for each eOrderLineBefore <> delete */
  end. /* transaction */
  run detachDataSources (input-output dataset dsOrder by-reference). 
end. /* saveChanges */    

procedure destroyObject:
  delete procedure this-procedure.  
end.     

       