/*------------------------------------------------------------------------------
 File        : beOrder.i
 Description : Business Entity for Order
 Notes       : This is a sample (prototype) for illustrative purposes only 
              -  Service as property (default to source in main) 
              -  DataAccess start and stop managed by service  
              -  This entity has no super procedure for demo/doc/lab purpose.                                                
------------------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
{schema/dsOrder.i} 
/* ************ Property Storage ************************************/
define variable ghService as handle no-undo.       
define variable ghDataAccessObject as handle no-undo.       
  
/* ***************************  ProtoTypes **************************** */

function getService  returns handle private  
  (  ) forward.

function setService  returns logical 
  ( phHandle as handle ) forward.


/* ************ Main Block ****************************************/  
      
ghService = source-procedure.

/* ************ Functions ****************************************/                           

/* private   */ 
function getDataAccessObject returns handle private
  (  ):
  if not valid-handle(ghDataAccessObject) then  
    ghDataAccessObject = dynamic-function('startDataAccess' in getService(),
                                           this-procedure).
  return ghDataAccessObject.                                           
end.       

function getService  returns handle private 
  (  ) .
  return ghService.
end.

function setService  returns logical 
  ( phHandle as handle ) :
  ghService = phHandle.
  return true.
end.
        
/* ************ Procedures ****************************************/        
procedure fetchDataset:
/*------------------------------------------------------------------------------
  Purpose:  Fetch dataset  
 -----------------------------------------------------------------------------*/
  define output parameter dataset for dsOrder.
end. /* fetchData */ 

procedure fetchData:
/*------------------------------------------------------------------------------
  Purpose:  Fetch data and/or definitions  
               
Parameters:  
     pcTables       - Tables for which to retrieve data or definitions
     pcQueries      - List of queries that corresponds to the requested tables
      pcNumRecords   - Batch sizes
     pcContext      - context
     phDataSet      - Prodataset handle (not dataset-handle) 
 
 -----------------------------------------------------------------------------*/
  define input        parameter pcTables       as character no-undo.
  define input        parameter pcQueries      as character no-undo.
  define input        parameter pcRequests     as character no-undo.
  define input        parameter piBatchSize    as integer   no-undo.
  define input-output parameter pcContext      as character no-undo.
  define output       parameter dataset for dsOrder.
  
  /* don't fetch for definition request */
  if pcTables > '' then
  do: 
    
    run fetchData in getDataAccessObject()
                    (pcTables,
                     pcQueries,
                     pcRequests,
                     piBatchsize,
                     input-output pcContext,
                     output dataset dsOrder by-reference).
                     
    buffer eOrderLine:set-callback-procedure('row-update','eOrderLineRowUpdate').
    /* when the be is running in the same session as client then 
       ensure that this change does not set roe-state */
   /* buffer eOrder:table-handle:tracking-changes=false.*/ 
    for each eOrder .
      eOrder.OrderTotal = 0.
      for each eOrderLine where eOrderLine.OrderNum = eOrder.OrderNum :
        eOrder.OrderTotal = eOrder.OrderTotal + eOrderLine.ExtendedPrice.
      end.
    end.     
  end.                                         
  
end. /* fetchData */ 

procedure saveChanges:
/*------------------------------------------------------------------------------
   Purpose:  Save data                
Parameters: phDataSet      - Prodataset with changes   
----------------------------------------------------------------------------*/
  define input-output parameter dataset for dsOrder.
  
  for each eOrderLine:
    find eOrderLineBefore 
         where rowid(eOrderLineBefore) = buffer eOrderLine:before-rowid no-error.
    if available eOrderLineBefore then
    do:  
      /* If the customer doubled the quantity ordered, then
	       increase the discount by 20%. */
      if  eOrderLine.Qty >= (eOrderLineBefore.Qty * 2) 
      and eOrderLine.Discount = eOrderLineBefore.Discount then 
        eOrderLine.Discount = eOrderLineBefore.Discount * 1.2.
    
      else if eOrderLine.Qty <= (eOrderLineBefore.Qty * .5) then 
        assign  dataset dsOrder:error = yes
                buffer eOrderLine:error = yes 
                buffer eOrderLine:error-string = 
                   "Line " + STRING(eOrderLine.LineNum) + 
                   ": You can't drop the Qty that much!".
    end.  
  end.
end.  

procedure destroyObject:
  dynamic-function('stopDataAccess' in getService(),this-procedure). 
  delete procedure this-procedure.  
end.     

/* This validation assumes that all items are filled in the dataset 
   (The adm2 also assumes this when the relation is defined as reposition ) */
procedure eOrderLineRowUpdate:
  /* row level triggers cannot update parent if client can undo */
  define input parameter dataset for dsOrder.
 
  find eItem where eItem.itemnum = eOrderLine.Itemnum no-error.
  
  if not available eItem then 
  do:
    buffer eOrderLine:error = true.
    buffer eOrderLine:error-string = 'Item does not exist'. 
  end.                     
end.     

       
       
       