/*------------------------------------------------------------------------------
 File        : beOrder.i
 Description : Business Entity for Order
 Notes       : This is a sample (prototype) for illustrative purposes only 
              -  Service as property (default to source in main) 
              -  DataAccess start and stop managed by service                                                
------------------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
/* defines dataset include and loads beSupport as super */ 
{support/beentity.i Order}.  
/* ************ Main Block ****************************************/  

/* ************ Functions ****************************************/                           
         
/* ************ Procedures ****************************************/        
procedure receiveData:
/*------------------------------------------------------------------------------
  Purpose:  receive data event                 
 -----------------------------------------------------------------------------*/
  define input-output parameter dataset for dsOrder.
  
  for each eOrder:
    eOrder.OrderTotal = 0.
    for each eOrderLine where eOrderLine.OrderNum = eOrder.OrderNum :
      eOrder.OrderTotal = eOrder.OrderTotal + eOrderLine.ExtendedPrice.
    end.
  end.                                           
end. 

procedure validateData:
/*------------------------------------------------------------------------------
   Purpose: Save data                
Parameters: phDataSet      - Prodataset with changes   
      Note:
----------------------------------------------------------------------------*/
  define input-output parameter dataset for dsOrder.
  
  for each eOrderLine:
    find eOrderLineBefore 
         where rowid(eOrderLineBefore) = buffer eOrderLine:before-rowid no-error.
    if available eOrderLineBefore then
    do:  
      /* If the customer doubled the quantity ordered, then
	       increase the discount by 20%. */
      if  eOrderLine.Qty >= (eOrderLineBefore.Qty * 2) 
      and eOrderLine.Discount = eOrderLineBefore.Discount then 
        eOrderLine.Discount = eOrderLineBefore.Discount * 1.2.
    
      else if eOrderLine.Qty <= (eOrderLineBefore.Qty * .5) then 
      do:
        assign  dataset dsOrder:error = yes
                buffer eOrderLine:error = yes 
                buffer eOrderLine:error-string = 
                   "Line " + string(eOrderLine.LineNum) + 
                   ": You can't drop the Qty that much!".
      end.
    end.  
  end.
end.  
   
/* This validation assumes that all items are filled in the dataset 
   (The adm2 also assumes this when the relation is defined as reposition ) */
procedure eOrderLineRowUpdate:
  /* row level triggers cannot update parent if client can undo */
  define input parameter dataset for dsOrder.
 
  find eItem where eItem.itemnum = eOrderLine.Itemnum no-error.
  
  if not available eItem then 
  do:
    buffer eOrderLine:error = true.
    buffer eOrderLine:error-string = 'Item does not exist'. 
  end.                     
end.     

       
       
       