/*************************************************************/
/* Copyright (c) 2007 by Progress Software Corporation      */
/*************************************************************/
/*------------------------------------------------------------------------------
 File        : Data access/service proxy 
 Description : Example/prototype code
 Notes       : Implements data access request interface allowing a distributed 
               BE to use it instead of the DAO.
            -  Uses Appbuilder appserver aware support 
               Use ProTools appserver paramter tool to define the 
               AppService  
 ---------------------------------------------------------------------------- */
/*-------------- Definitions --------------------------------------------------*/

define variable gcAppService as character no-undo.
define variable gcEntityName as character no-undo.

{adecomm/appserv.i}

procedure fetchData:
/*------------------------------------------------------------------------------
  Purpose: Fetch data from another session.
               
Parameters:  
     phDataSet      - dataset-handle
     pcContext      - context    
     pcTables       - Tables for which to retrieve data or definitions
     pcQueries      - List of queries that corresponds to the requested tables
     pcRequest      - The actual request for the tables   
     piNumRecords   - Batch sizes
 -----------------------------------------------------------------------------*/
  define input        parameter pcTables       as character no-undo.
  define input        parameter pcQueries      as character no-undo.
  define input        parameter pcRequests     as character no-undo.
  define input        parameter piBatchSize    as integer   no-undo.
  define input-output parameter pcContext      as character no-undo.
  define input-output parameter dataset-handle phDataset.
 
  define variable hConnection as handle    no-undo.
  hConnection = dynamic-function('getConnection' in target-procedure). 

  if valid-handle(hConnection) then 
  do:
    run support/fetchdata.p on hConnection 
                        (dynamic-function('getEntityName' in target-procedure),
                         pcTables,
                         pcQueries,
                         pcRequests,
                         piBatchSize,
                         input-output pcContext,
                         output dataset-handle phDataset append) no-error.
    if error-status:error then 
      return error if return-value > '' then return-value else error-status:get-message(1).                  
  end.
  else 
    return error dynamic-function('getConnectError' in target-procedure).
                      
end. /* retrieveData */

procedure saveChanges:
/*------------------------------------------------------------------------------
  Purpose:  Save changes in another session                
Parameters: dataset-handle  
-----------------------------------------------------------------------------*/
  define input-output parameter dataset-handle phDataset.

  define variable hConnection as handle    no-undo.
  hConnection = dynamic-function('getConnection' in target-procedure). 
  if valid-handle(hConnection) then
  do: 
    run support/savechanges.p on hConnection 
                     (dynamic-function('getEntityName' in target-procedure),
                      input-output dataset-handle phDataset) no-error.
    if error-status:error then 
      return error if return-value > '' then return-value else error-status:get-message(1).                  
  end.                    
  else 
    return error dynamic-function('getConnectError' in target-procedure).
                         
end. /* saveChanges */ 
 
procedure destroyObject:
  delete procedure target-procedure.  
end.     

function getAppService returns character 
  ( ):
  return gcAppService.
end.

function setAppService returns logical 
  (pcAppService as character ):
  gcAppService = pcAppService. 
end.        

function getEntityName returns character 
  ( ):
  return gcEntityName. 
end.        

function setEntityName returns logical 
  (pcName as character ):
  gcEntityName = pcName. 
end.        

function getConnection returns handle ():
  define variable hAppService as handle    no-undo.
  define variable cAppService as character no-undo. 
  cAppService = dynamic-function('getAppService' in target-procedure).
  if cAppService = "" then 
    hAppService = session:handle.
  else do:   
    hAppService = dynamic-function('getServiceHandle':U IN appSrvUtils,
                                    cAppService).
  
    if not valid-handle(hAppService) 
    or (hAppService <> session:handle and not hAppService:connected()) then  
      run appServerConnect in appSrvUtils(cAppService, no, '', output hAppService).
  end.
  return hAppService.
end.

function getConnectError returns character 
  ():
  return 'Could not connect to AppService' 
                 + dynamic-function('getAppService' in target-procedure).                
end.         

/* used by locateProcedure (see locateprocedure.i) */
procedure isRunning:
  define output parameter phThis as handle no-undo.
  phThis = this-procedure.
end.  