/*************************************************************/
/* Copyright (c) 2007 by Progress Software Corporation      */
/*************************************************************/
/*------------------------------------------------------------------------
    File        : service.p
    Purpose     : Entity manager and request service               
  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
/* keeps track of started entities */
define temp-table ttEntity no-undo
  field ProcHandle as handle
  field DAOHandle  as handle
  field Name       as character
  field NumClients as integer
  index ProcName as unique name
  index ProcHandle as unique Prochandle 
  index DAOHandle DAOhandle.
  
/* property storage */  
define variable glAllowSource   as logical   no-undo.
define variable gcPath          as character no-undo.
define variable gcOperatingMode as character no-undo.
/* ************************  Function Prototypes ***************** */

/* ------ Properties ------------------------ */
function getAllowSource returns logical 
   ( ) forward:  

function setAllowSource returns logical 
   (plAllow as logical ) forward:  

function getPath returns character  
   ( ) forward:  

function setPath returns logical   
   ( pcPath as character) forward:  

function getOperatingMode returns character 
   ( ) forward. 

function setOperatingMode returns logical 
   (pcMode as char ) forward. 

function getTypePath returns character  
   (pcType as char) forward:

/* ------ Private functions ------------------------ */
function getEntity returns handle private  
   ( pcName as character) forward.  

function getDataAccess returns handle private  
   ( pcName as character) forward.  

function getPhysicalName returns character private 
    ( pcType as character,
      pcName as character) forward.  

function startProcedure returns handle private 
   ( pcType as character,
     pcName as character) forward.  

/* ***************************  Main Block  *************************** */
/*set default props */
setAllowSource(true).
setPath('service').
 
/* **********************  Internal Procedures  *********************** */
procedure destroyObject:
  delete object this-procedure.
end.

procedure fetchData:
/*------------------------------------------------------------------------------
   Purpose:  Fetch data and/or definitions for a Business Entity.                 
Parameters:  
     pcEntity       - Business Entity 
     pcTables       - Tables for which to retrieve data or definitions
     pcQueries      - List of queries that corresponds to the requested tables
     pcRequest      - The actual request for the tables   
     phDataSet      - Prodataset handle 
     pcContext      - context    
 -----------------------------------------------------------------------------*/
 define input        parameter pcEntity       as character no-undo.
 define input        parameter pcTables       as character no-undo.
 define input        parameter pcQueries      as character no-undo.
 define input        parameter pcRequests     as character no-undo.
 define input        parameter piNumRecords   as integer   no-undo.
 define input-output parameter pcContext      as character no-undo.
 define input-output parameter phDataSet      as handle    no-undo.
  
 define variable hEntity as handle    no-undo.

 hEntity = getEntity(pcEntity).
 
 if valid-handle(hEntity) then
 do:    
   run fetchData in hEntity
                   (pcTables,
                    pcQueries,
                    pcRequests,
                    piNumRecords,
                    input-output pcContext,
                    input-output dataset-handle phDataset by-reference) no-error.
                   
    if error-status:error then 
      return error if return-value > '' then return-value else error-status:get-message(1) .
  end.
 else 
   return error 'Business Entity "' + pcEntity + '" does not exist or did not start'.   


end. /* fetchData */

procedure fetchDataset:
/*------------------------------------------------------------------------------
   Purpose:  Fetch local definitions for a Business Entity.                 
Parameters:  
     pcEntity -  Business Entity 
     Notes: in anticipation of distribution 
            This does NOT go to server.
            A dynamic BE should return invalid (at least until a fetchData 
            request has been issued)
 -----------------------------------------------------------------------------*/
  define input  parameter pcEntity       as character no-undo.
  define output parameter phDataset      as handle    no-undo.
  
  define variable hEntity as handle    no-undo.

  hEntity = getEntity(pcEntity).
 
  if valid-handle(hEntity) then
  do:
    run fetchDataset in hEntity
               (output dataset-handle phDataset by-reference) no-error.
  end.
  else 
    return error 'Business Entity "' + pcEntity + '" does not exist or did not start'.   
  
  if error-status:error then return error if return-value > '' then return-value else error-status:get-message(1) .
 
end. /* fetchDataset */


procedure saveChanges:
 /*-----------------------------------------------------------------------------
  Purpose:  Save changes for a Business Entity 
               
Parameters:  
     pcEntity       - Business Entity 
     phDataSet      - Prodataset handle (not dataset-handle)
----------------------------------------------------------------------------*/
 define input        parameter pcEntity  as character no-undo.
 define input-output parameter phDataset as handle. 
   
 define variable hEntity as handle    no-undo.
 hEntity = getEntity(pcEntity).
 if valid-handle(hEntity) then
   run saveChanges in hEntity (input-output dataset-handle phDataset by-reference).
 else 
   return error 'Business Entity "' + pcEntity + '" does not exist or did not start'.
 
 if error-status:error then return error if return-value > '' then return-value else error-status:get-message(1) .
     
end.

procedure stopEntity:
  define input parameter pcName as character no-undo.
  
  define buffer btEntity for ttEntity.
  find btEntity where btEntity.Name = pcName no-error. 
  if available btEntity then 
  do:
    run destroyObject in btEntity.ProcHandle.
    delete btEntity. 
  end.    
end.
   
/* ************************  Function Implementations ***************** */
function getEntity returns handle private 
   ( pcName    as character): 
  
  define buffer btEntity for ttEntity.

  define variable hProc as handle    no-undo.
  
  find btEntity where btEntity.Name = pcName no-error. 
  if not available btEntity or not valid-handle(btEntity.ProcHandle) then
  do:
    hProc = startProcedure('BE',pcName).
    if valid-handle(hProc) and not available btEntity then
    do:
      create btEntity.      
      assign btEntity.Name = pcName.
    end.  
    dynamic-function("setService" in hProc,target-procedure).
    /* client and local session assumes rich client (= row level triggers)*/
    if lookup(getOperatingMode(),"client,local") > 0 then 
      dynamic-function("setHasClient" in hProc,true).
    btEntity.ProcHandle = hProc.
  end.
  return if available btEntity then btEntity.ProcHandle else ?.         
end function. /* getEntity */

function getDataAccess returns handle private 
   ( pcName as character):  
  define buffer btEntity for ttEntity.

  define variable hProc as handle    no-undo.
  
  find btEntity where btEntity.Name = pcName no-error. 
  if available btEntity and not valid-handle(btEntity.DAOHandle) then
    btEntity.DAOHandle = startProcedure('DA',pcName).
                                         
  return if available btEntity then btEntity.DAOHandle else ?. 
                    
end function. /* getDataAccess  */

function getPhysicalName returns character private 
   ( pcType as character, 
     pcName as character): 
  /* don't allow name client name to reference affect casing 
     (does not allow multi casing .. ) */     
  if getTypePath(pcType) = '' then       
    return lc(pcType) + caps(substring(pcName,1,1)) 
                      + lc(substring(pcName,2)). 
  else 
    return lc(pcName).
                             
end function. /* getPhysicalName */

/***  startDataAceess is to be called from entity with handle
      Could have used name, but that would make it look very public.. 
      (feel free to add function prototype) **/
function startDataAccess returns handle   
   ( phEntity as handle):  
  define buffer btEntity for ttEntity.
  find btEntity where btEntity.ProcHandle = phEntity no-error.
  if available btEntity then
    return getDataAccess(btEntity.Name).
end. /* startDataAccess */

/***  stopDataAceess is to be called from entity with handle
      Could have used name, but that would make it look very public..  
      (feel free to add function prototype) **/
function stopDataAccess returns logical   
   ( phEntity as handle):
  define buffer btEntity for ttEntity.
  define buffer btDAO for ttEntity.
  find btEntity where btEntity.ProcHandle = phEntity no-error.
  if available btEntity then
  do: 
    /* allow sharing of data access objects only destroy if not in use */
    if not can-find(first btDAO where btDao.DAOHandle = btEntity.DAOHandle
                                and   btDAo.ProcHandle <> btEntity.DAOHandle) then
      run destroyObject in btEntity.DAOHandle.
    btEntity.DAOHandle = ?. 
    return true.  
  end.        
  return false.
end. /* startDataAccess */

function startProcedure returns handle private 
   ( pcType   as character,
     pcName   as character):
       
  define variable hdl       as handle    no-undo. 
  define variable cDir      as character no-undo.
  define variable cSubDir   as character no-undo.
  
  /* client use da proxy */
  if pcType = "DA" and getOperatingMode() = 'client' then 
  do:
    run support/thinproxy.w persistent set hdl.
    dynamic-function('setEntityName' in hdl,pcName).
  end.       
  else 
  do:
    assign
      cDir    = getPath()
      cSubDir = getTypePath(pcType)
      pcName  = getPhysicalName(pcType,pcName)
      pcName  = (if cDir > '' then cDir + '/' else '')
              + (if cSubDir > '' then cSubdir + '/' else '')
              + pcName 
              + (if getAllowSource() then '.p' else '.r')         
      pcName = search(pcName). 
  
    if pcName <> ? then                   
      run value(pcName) persistent set hdl.
      
    else if pcType = 'BE' and getOperatingMode() = 'client'  then
    do: 
      run support/thinbe.w persistent set hdl.
      dynamic-function('setEntityName' in hdl,pcName).
    end.
    /** Data access could be generic 
       (need to add EntityName property to daSupport for this to work)
    else if pcType = 'DA' then
    do: 
      run support/thinda.w persistent set hdl.
      dynamic-function('setEntityName' in hdl,pcName).
    end. 
    **/ 
  end.
  return hdl.
end function. /* startProcedure */

/*----------- Properties ----------------------*/
function setPath returns logical  
   ( pcPath as character): 
  gcPath = pcPath.  
  return true.   
end.  /* setPath */

function getPath returns character  
   ( ) : 
   return gcPath.   
end. /* getPath */

function getTypePath returns character  
   (pcType as char) :
   /*   
   case pcType.
     when 'be' then 
       return 'entity'.
     when 'da' then 
       return 'dataaccess'.
     when 'sc' then   
       return 'datasource'.
   end.
   */   
   return ''.
end. /* getTypePath */

function setAllowSource returns logical  
   ( plAllow as logical): 
  glAllowSource = plAllow.  
  return true.   
end. /* setAllowSource */

function getAllowSource returns logical  
   ( ) : 
  return glAllowSource.   
end. /* getAllowSource */

function getOperatingMode returns character 
   ( ):  
  return gcOperatingMode.   
end.

function setOperatingMode returns logical 
   (pcMode as char ):  
  gcOperatingMode = pcMode.   
  return true.      
end.

FUNCTION getObjectType RETURNS CHARACTER
  (  ) :
/*------------------------------------------------------------------------------
  Purpose: To be used by designtime super destroy  
    Notes:  
------------------------------------------------------------------------------*/
  RETURN "Service". 
END FUNCTION.

/* used by locateProcedure (see runpersistent.i) */
procedure isRunning:
  define output parameter phThis as handle no-undo.
  phThis = this-procedure.
end.  